@@ -5,22 +5,14 @@ import warnings
 from abc import ABC
 from abc import ABCMeta
 from collections import defaultdict
-from inspect import isclass
-from tracemalloc import start
 from typing import Any
-from typing import Dict
 from typing import Generic
-from typing import List
-from typing import Set
-from typing import Tuple
-from typing import Type
 
 from geneticengine.core.decorators import get_gengy
 from geneticengine.core.utils import all_init_arguments_typed
 from geneticengine.core.utils import get_arguments
 from geneticengine.core.utils import get_generic_parameter
 from geneticengine.core.utils import get_generic_parameters
-from geneticengine.core.utils import has_arguments
 from geneticengine.core.utils import is_abstract
 from geneticengine.core.utils import is_annotated
 from geneticengine.core.utils import is_generic
@@ -36,17 +28,6 @@ class InvalidGrammarException(Exception):
     pass
 
 
-class EvolveGrammar:
-    """Class to be assigned to the evolve_grammar parameter in the GP class.
-
-    Args:
-        learning_rate (float): The learning rate with which the grammar is updated each generation (default as in https://arxiv.org/pdf/2103.08389.pdf = 0.01).
-    """
-
-    def __init__(self, learning_rate=0.01) -> None:
-        self.learning_rate = learning_rate
-
-
 class Grammar:
     starting_symbol: type
     alternatives: dict[type, list[type]]
@@ -87,11 +68,7 @@ class Grammar:
             elif all_init_arguments_typed(c):
                 # Raise a warning if there are annotated elements, but no __init__ method.
                 d = {x[0]: x[1] for x in inspect.getmembers(c)}
-                if (
-                    "__annotations__" in d
-                    and d["__annotations__"]
-                    and d["__init__"].__qualname__ == "object.__init__"
-                ):
+                if "__annotations__" in d and d["__annotations__"] and d["__init__"].__qualname__ == "object.__init__":
                     warnings.warn(
                         f"Warning: class {c} looks like it should be a dataclass, but isn't.",
                     )
@@ -185,15 +162,11 @@ class Grammar:
 
         prods = ";".join(
             [
-                str(p.__name__)
-                + " -> "
-                + ("|".join([format(p) for p in self.alternatives[p]]))
+                str(p.__name__) + " -> " + ("|".join([format(p) for p in self.alternatives[p]]))
                 for p in self.alternatives
             ],
         )
-        return (
-            f"Grammar<Starting={self.starting_symbol.__name__},Productions=[{prods}]>"
-        )
+        return f"Grammar<Starting={self.starting_symbol.__name__},Productions=[{prods}]>"
 
     def get_all_symbols(self) -> tuple[set[type], set[type], set[type]]:
         """All symbols in the current grammar, including terminals."""
@@ -228,7 +201,7 @@ class Grammar:
 
         return max(list(map(dist, self.all_nodes)))
 
-    def preprocess(self):
+    def preprocess(self) -> None:
         """Computes distanceToTerminal via a fixpoint algorithm."""
         (keys, _, all_sym) = self.get_all_symbols()
         for s in all_sym:
@@ -262,40 +235,30 @@ class Grammar:
                         for prod in prods:
                             val = min(
                                 val,
-                                int(self.expansion_depthing)
-                                + self.distanceToTerminal[prod],
+                                int(self.expansion_depthing) + self.distanceToTerminal[prod],
                             )
                             old = self.abstract_dist_to_t[sym][prod]
                             if 1 < old:
                                 self.abstract_dist_to_t[sym][prod] = 1
                                 changed = True
                             if prod in self.abstract_dist_to_t:
-                                for subprod, dist in self.abstract_dist_to_t[
-                                    prod
-                                ].items():
+                                for subprod, dist in self.abstract_dist_to_t[prod].items():
                                     currdist = self.abstract_dist_to_t[sym][subprod]
                                     candidate = dist + 1
                                     if candidate < currdist:
-                                        self.abstract_dist_to_t[sym][
-                                            subprod
-                                        ] = candidate
+                                        self.abstract_dist_to_t[sym][subprod] = candidate
                                         changed = True
                         changed |= process_reachability(sym, prods)
                 else:
                     if is_terminal(sym, self.non_terminals):
-                        if (
-                            sym == int or sym == float or sym == str
-                        ) and not self.expansion_depthing:
+                        if (sym == int or sym == float or sym == str) and not self.expansion_depthing:
                             val = 0
                         else:
                             val = 1
                     else:
                         args = get_arguments(sym)
                         assert args
-                        val = max(
-                            1 + self.get_distance_to_terminal(argt)
-                            for (_, argt) in args
-                        )
+                        val = max(1 + self.get_distance_to_terminal(argt) for (_, argt) in args)
 
                         changed |= process_reachability(
                             sym,
@@ -341,22 +304,6 @@ class Grammar:
         self.preprocess()
         return self
 
-    def get_grammar_specifics(self):
-        '''
-        Returns grammar specifics:
-        (depth_min, depth_max), n_non_terminals, (n_prods_occurrences, n_recursive_prods)
-        '''
-        depth_min = self.get_min_tree_depth()
-        depth_max = self.get_max_node_depth()
-        n_non_terminals = len(self.alternatives)
-        n_prods_per_nt = list(map(lambda x: len(x), self.alternatives.values()))
-        n_prods_occurrences = dict()
-        for i in n_prods_per_nt:
-            n_prods_occurrences[i] = n_prods_occurrences.get(i, 0) + 1
-        n_prods_occurrences = {k: n_prods_occurrences[k] for k in sorted(n_prods_occurrences.keys())}
-        recursive_prods = [ r_prod for r_prod in self.recursive_prods if r_prod not in self.alternatives.keys()]
-        n_recursive_prods = len(recursive_prods)
-        return (depth_min, depth_max), (n_non_terminals), (n_prods_occurrences, n_recursive_prods)
 
 def extract_grammar(
     considered_subtypes: list[type],
@@ -369,8 +316,10 @@ def extract_grammar(
     such as Genetic Programming and Hill Climbing.
 
     Args:
-        nodes (list): A list of objects representing tree nodes. Make sure that any node can be produced be the starting symbol.
-        starting_symbol (object): The starting symbol of each tree. Makes sure every generated tree by the returned grammar starts with this symbol. Make sure that the starting symbol can produce any object of nodes.
+        nodes (list): A list of objects representing tree nodes. Make sure that any node can be produced be the starting
+            symbol.
+        starting_symbol (object): The starting symbol of each tree. Makes sure every generated tree by the returned
+            grammar starts with this symbol. Make sure that the starting symbol can produce any object of nodes.
 
     Returns:
         The grammar
