@@ -1,14 +1,19 @@
 from __future__ import annotations
 
 from typing import Any
-from typing import Callable
 from typing import TypeVar
 
 from geneticengine.core.grammar import Grammar
+from geneticengine.core.problems import Problem
 from geneticengine.core.random.sources import Source
+from geneticengine.core.representations.api import CrossoverOperator
+from geneticengine.core.representations.api import MutationOperator
 from geneticengine.core.representations.api import Representation
-from geneticengine.core.representations.tree.initialization_methods import Initialization_Method, PI_Grow, Random_Production, mk_save_init
-from geneticengine.core.representations.tree.utils import GengyList
+from geneticengine.core.representations.tree.initializations import (
+    InitializationMethodType,
+)
+from geneticengine.core.representations.tree.initializations import mk_save_init
+from geneticengine.core.representations.tree.initializations import pi_grow_method
 from geneticengine.core.representations.tree.utils import relabel_nodes
 from geneticengine.core.representations.tree.utils import relabel_nodes_of_trees
 from geneticengine.core.tree import TreeNode
@@ -17,37 +22,40 @@ from geneticengine.core.utils import get_generic_parameter
 from geneticengine.core.utils import has_annotated_crossover
 from geneticengine.core.utils import has_annotated_mutation
 from geneticengine.core.utils import is_abstract
+from geneticengine.core.evaluators import Evaluator
 from geneticengine.exceptions import GeneticEngineError
 
 T = TypeVar("T")
 
 
-
-
 def random_node(
     r: Source,
     g: Grammar,
     max_depth: int,
     starting_symbol: type[Any] | None = None,
-    method=PI_Grow().tree_init_method,
+    method: InitializationMethodType = pi_grow_method,
 ):
-    if starting_symbol is None:
-        starting_symbol = g.starting_symbol
+    starting_symbol = starting_symbol if starting_symbol else g.starting_symbol
     return method(r, g, max_depth, starting_symbol)
 
 
-
-
-def random_individual(r: Source, g: Grammar, max_depth: int = 5, method = PI_Grow().tree_init_method) -> TreeNode:
+def random_individual(
+    r: Source,
+    g: Grammar,
+    max_depth: int = 5,
+    method: InitializationMethodType = pi_grow_method,
+) -> TreeNode:
     try:
         assert max_depth >= g.get_min_tree_depth()
-    except:
+    except AssertionError:
         if g.get_min_tree_depth() == 1000000:
             raise GeneticEngineError(
-                f"Grammar's minimal tree depth is {g.get_min_tree_depth()}, which is the default tree depth. It's highly like that there are nodes of your grammar than cannot reach any terminal.",
+                f"""Grammar's minimal tree depth is {g.get_min_tree_depth()}, which is the default tree depth.
+                 It's highly like that there are nodes of your grammar than cannot reach any terminal.""",
             )
         raise GeneticEngineError(
-            f"Cannot use complete grammar for individual creation. Max depth ({max_depth}) is smaller than grammar's minimal tree depth ({g.get_min_tree_depth()}).",
+            f"""Cannot use complete grammar for individual creation. Max depth ({max_depth})
+            is smaller than grammar's minimal tree depth ({g.get_min_tree_depth()}).""",
         )
     ind = random_node(r, g, max_depth, g.starting_symbol, method)
     assert isinstance(ind, TreeNode)
@@ -66,26 +74,13 @@ def mutate_inner(
     counter = i.gengy_weighted_nodes if depth_aware_mut else i.gengy_nodes
     if counter > 0:
         c = r.randint(0, counter - 1)
-        if (
-            c == 0
-            or (c <= i.gengy_distance_to_term and depth_aware_mut)
-            or force_mutate
-        ):
+        if c == 0 or (c <= i.gengy_distance_to_term and depth_aware_mut) or force_mutate:
             # If Metahandler mutation exists, the mutation process is different
-            args_with_specific_mutation = [
-                has_annotated_mutation(arg[1]) for arg in get_arguments(i)
-            ]
-            if any(args_with_specific_mutation):
-                mutation_possibilities = len(args_with_specific_mutation)
-                mutation_choice = r.randint(
-                    0,
-                    mutation_possibilities - 1,
-                )
-                (index, arg_to_be_mutated) = [
-                    (kdx, arg[1])
-                    for kdx, arg in enumerate(get_arguments(i))
-                    if args_with_specific_mutation[kdx]
-                ][mutation_choice]
+            if any(has_annotated_mutation(arg[1]) for arg in get_arguments(i)):
+                options = [(kdx, arg[1]) for kdx, arg in enumerate(get_arguments(i)) if has_annotated_mutation(arg[1])]
+                index = r.randint(0, len(options) - 1)
+                (index, arg_to_be_mutated) = options[index]
+
                 args = list(i.gengy_init_values)
                 args[index] = arg_to_be_mutated.__metadata__[0].mutate(  # type: ignore
                     r,
@@ -93,17 +88,20 @@ def mutate_inner(
                     random_node,
                     max_depth - 1,
                     get_generic_parameter(arg_to_be_mutated),
-                    method=Random_Production().tree_init_method,
                     current_node=args[index],
                 )
                 mk = mk_save_init(type(i), lambda x: x)(*args)
                 return mk
 
+            replacement = None
             for _ in range(5):
-                replacement = random_node(r, g, max_depth, ty, method=Random_Production().tree_init_method)
-                if replacement != i:
-                    break
-            return replacement
+                try:
+                    replacement = random_node(r, g, max_depth, ty)
+                    if replacement != i:
+                        break
+                except GeneticEngineError:
+                    pass
+            return replacement if replacement else i
         else:
             if is_abstract(ty) and g.expansion_depthing:
                 max_depth -= g.abstract_dist_to_t[ty][type(i)]
@@ -113,11 +111,7 @@ def mutate_inner(
             for idx, (_, field_type) in enumerate(get_arguments(i)):
                 child = args[idx]
                 if hasattr(child, "gengy_nodes"):
-                    count = (
-                        child.gengy_weighted_nodes
-                        if depth_aware_mut
-                        else child.gengy_nodes
-                    )
+                    count = child.gengy_weighted_nodes if depth_aware_mut else child.gengy_nodes
                     if c <= count:
                         mi = mutate_inner(
                             r,
@@ -135,11 +129,15 @@ def mutate_inner(
             mk = mk_save_init(i, lambda x: x)(*args)
             return mk
     else:
+        rn = None
         for _ in range(5):
-            rn = random_node(r, g, max_depth, ty, method=Random_Production().tree_init_method)
-            if rn != i:
-                break
-        return rn
+            try:
+                rn = random_node(r, g, max_depth, ty)
+                if rn != i:
+                    break
+            except GeneticEngineError:
+                pass
+        return rn if rn else i
 
 
 def mutate_specific_type_inner(
@@ -230,19 +228,8 @@ def mutate(
     i: TreeNode,
     max_depth: int,
     target_type: type,
-    specific_type: type | None = None,
     depth_aware_mut: bool = False,
 ) -> Any:
-    if specific_type:
-        return mutate_specific_type(
-            r,
-            g,
-            i,
-            max_depth,
-            target_type,
-            specific_type,
-            depth_aware_mut,
-        )
     new_tree = mutate_inner(r, g, i, max_depth, target_type, False, depth_aware_mut)
     relabeled_new_tree = relabel_nodes_of_trees(new_tree, g)
     return relabeled_new_tree
@@ -294,15 +281,9 @@ def crossover_inner(
     counter = i.gengy_weighted_nodes if depth_aware_co else i.gengy_nodes
     if counter > 0:
         c = r.randint(0, counter - 1)
-        if (
-            c == 0
-            or (c <= i.gengy_distance_to_term and depth_aware_co)
-            or force_crossover
-        ):
+        if c == 0 or (c <= i.gengy_distance_to_term and depth_aware_co) or force_crossover:
             replacement = None
-            args_with_specific_crossover = [
-                has_annotated_crossover(arg[1]) for arg in get_arguments(i)
-            ]
+            args_with_specific_crossover = [has_annotated_crossover(arg[1]) for arg in get_arguments(i)]
             if any(args_with_specific_crossover):
                 crossover_possibilities = len(args_with_specific_crossover)
                 crossover_choice = r.randint(
@@ -313,32 +294,31 @@ def crossover_inner(
                 if not options:
                     pass  # Replace whole node
                 else:
-                    (index, arg_to_be_crossovered) = [
-                        (kdx, arg)
-                        for kdx, arg in enumerate(get_arguments(i))
-                        if args_with_specific_crossover[kdx]
-                    ][crossover_choice]
+                    (index, arg_to_be_crossovered) = [(kdx, arg) for kdx, arg in enumerate(get_arguments(i))][
+                        crossover_choice
+                    ]
                     args = list(i.gengy_init_values)
-                    args[index] = (
-                        arg_to_be_crossovered[1]
-                        .__metadata__[0]  # type: ignore
-                        .crossover(
-                            r,
-                            g,
-                            options,
-                            arg_to_be_crossovered[0],
-                            ty,
-                            current_node=args[index],
+                    if has_annotated_crossover(arg_to_be_crossovered[1]):
+                        args[index] = (
+                            arg_to_be_crossovered[1]
+                            .__metadata__[0]  # type: ignore
+                            .crossover(
+                                r,
+                                g,
+                                options,
+                                arg_to_be_crossovered[0],
+                                ty,
+                                current_node=args[index],
+                            )
                         )
-                    )
-                    return mk_save_init(type(i), lambda x: x)(*args)
+                        return mk_save_init(type(i), lambda x: x)(*args)
 
             options = list(find_in_tree(g, ty, o, max_depth))
             if options:
                 replacement = r.choice(options)
             if replacement is None:
                 for _ in range(5):
-                    replacement = random_node(r, g, max_depth, ty, method=Random_Production().tree_init_method)
+                    replacement = random_node(r, g, max_depth, ty)
                     if replacement != i:
                         break
 
@@ -352,11 +332,7 @@ def crossover_inner(
             for idx, (field, field_type) in enumerate(get_arguments(i)):
                 child = args[idx]
                 if hasattr(child, "gengy_nodes"):
-                    count = (
-                        child.gengy_weighted_nodes
-                        if depth_aware_co
-                        else child.gengy_nodes
-                    )
+                    count = child.gengy_weighted_nodes if depth_aware_co else child.gengy_nodes
                     if c <= count:
                         args[idx] = crossover_inner(
                             r,
@@ -474,8 +450,13 @@ def crossover(
     specific_type: type | None = None,
     depth_aware_co: bool = False,
 ) -> tuple[TreeNode, TreeNode]:
-    """
-    Given the two input trees [p1] and [p2], the grammar and the random source, this function returns two trees that are created by crossing over [p1] and [p2]. The first tree returned has [p1] as the base, and the second tree has [p2] as a base.
+    """Given the two input trees [p1] and [p2], the grammar and the random
+    source, this function returns two trees that are created by crossing over.
+
+    [p1] and [p2].
+
+    The first tree returned has [p1] as the base, and the second tree
+    has [p2] as a base.
     """
     if specific_type:
         new_tree1 = crossover_specific_type(
@@ -527,44 +508,170 @@ def crossover(
     return relabeled_new_tree1, relabeled_new_tree2
 
 
-class TreeBasedRepresentation(Representation[TreeNode]):
-    """This class represents the tree representation of an individual.
-    In this approach, the genotype and the phenotype are exactly the same."""
+class DefaultTBMutation(MutationOperator[TreeNode]):
+    """Selects a random node, and generates a new replacement."""
 
-    def __init__(self, depth = None, method: Initialization_Method = PI_Grow()) -> None:
-        self.depth = depth
-        self.method = method
-        
-    def create_individual(self, r: Source, g: Grammar, depth: int) -> TreeNode:
-        return random_individual(r, g, depth, self.method.tree_init_method)
+    def __init__(self, depth_aware: bool = False):
+        """
+        Args:
+            depth_aware (bool): whether the mutation should be depth-aware.
 
-    def mutate_individual(
+        """
+        self.depth_aware = depth_aware
+
+    def mutate(
         self,
-        r: Source,
-        g: Grammar,
-        ind: TreeNode,
-        depth: int,
-        ty: type,
-        specific_type: type | None = None,
-        depth_aware_mut: bool = False,
+        genotype: TreeNode,
+        problem: Problem,
+        evaluator: Evaluator,
+        representation: Representation,
+        random_source: Source,
+        index_in_population: int,
+        generation: int,
     ) -> TreeNode:
-        new_ind = mutate(r, g, ind, depth, ty, specific_type, depth_aware_mut)
-        return new_ind
+        assert isinstance(representation, TreeBasedRepresentation)
+        return mutate(
+            random_source,
+            representation.grammar,
+            genotype,
+            representation.max_depth,
+            representation.grammar.starting_symbol,
+        )
 
-    def crossover_individuals(
+
+class TypeSpecificTBMutation(DefaultTBMutation):
+    """Selects a random node of a given type, and generates a new
+    replacement."""
+
+    def __init__(self, specific_type: type, depth_aware: bool):
+        super().__init__(depth_aware=depth_aware)
+
+        self.specific_type = specific_type
+
+    def mutate(
+        self,
+        genotype: TreeNode,
+        problem: Problem,
+        evaluator: Evaluator,
+        representation: Representation,
+        random_source: Source,
+        index_in_population: int,
+        generation: int,
+    ) -> TreeNode:
+        assert isinstance(representation, TreeBasedRepresentation)
+        return mutate_specific_type(
+            random_source,
+            representation.grammar,
+            genotype,
+            representation.max_depth,
+            representation.grammar.starting_symbol,
+            self.specific_type,
+            self.depth_aware,
+        )
+
+
+class DefaultTBCrossover(CrossoverOperator[TreeNode]):
+    """Selects a sub-tree from one parent and replaces it with a compatible
+    tree from the other parent."""
+
+    def __init__(self, depth_aware: bool = False):
+        """
+        Args:
+            depth_aware (bool): whether the mutation should be depth-aware.
+
+        """
+        self.depth_aware = depth_aware
+
+    def crossover(
+        self,
+        g1: TreeNode,
+        g2: TreeNode,
+        problem: Problem,
+        representation: Representation,
+        random_source: Source,
+        index_in_population: int,
+        generation: int,
+    ) -> tuple[TreeNode, TreeNode]:
+        assert isinstance(representation, TreeBasedRepresentation)
+        return crossover(random_source, representation.grammar, g1, g2, representation.max_depth)
+
+
+class TypeSpecificTBCrossover(DefaultTBCrossover):
+    """Selects a sub-tree from one parent and replaces it with a compatible
+    tree from the other parent."""
+
+    def __init__(self, specific_type: type, depth_aware: bool):
+        super().__init__(depth_aware=depth_aware)
+
+        self.specific_type = specific_type
+
+    def crossover(
+        self,
+        g1: TreeNode,
+        g2: TreeNode,
+        problem: Problem,
+        representation: Representation,
+        random_source: Source,
+        index_in_population: int,
+        generation: int,
+    ) -> tuple[TreeNode, TreeNode]:
+        assert isinstance(representation, TreeBasedRepresentation)
+        t1 = crossover_specific_type(
+            random_source,
+            representation.grammar,
+            g1,
+            g2,
+            representation.max_depth,
+            representation.grammar.starting_symbol,
+            self.specific_type,
+            self.depth_aware,
+        )
+        t2 = crossover_specific_type(
+            random_source,
+            representation.grammar,
+            g2,
+            g1,
+            representation.max_depth,
+            representation.grammar.starting_symbol,
+            self.specific_type,
+            self.depth_aware,
+        )
+        return (t1, t2)
+
+
+class TreeBasedRepresentation(Representation[TreeNode, TreeNode]):
+    """This class represents the tree representation of an individual.
+
+    In this approach, the genotype and the phenotype are exactly the
+    same.
+    """
+
+    def __init__(self, grammar: Grammar, max_depth: int):
+        super().__init__(grammar, max_depth)
+
+    def create_individual(
         self,
         r: Source,
-        g: Grammar,
-        i1: TreeNode,
-        i2: TreeNode,
-        max_depth: int,
-        specific_type: type | None = None,
-        depth_aware_co: bool = False,
-    ) -> tuple[TreeNode, TreeNode]:
-        return crossover(r, g, i1, i2, max_depth, specific_type, depth_aware_co)
+        depth: int | None = None,
+        initialization_method: InitializationMethodType = pi_grow_method,
+        **kwargs,
+    ) -> TreeNode:
+        actual_depth = depth or self.max_depth
+        return random_individual(r, self.grammar, actual_depth, initialization_method)
 
-    def genotype_to_phenotype(self, g: Grammar, genotype: TreeNode) -> TreeNode:
+    def genotype_to_phenotype(self, genotype: TreeNode) -> TreeNode:
         return genotype
 
+    def phenotype_to_genotype(self, phenotype: Any) -> TreeNode:
+        """Takes an existing program and adapts it to be used in the right
+        representation."""
+        return relabel_nodes_of_trees(
+            phenotype,
+            self.grammar,
+        )
 
-treebased_representation = TreeBasedRepresentation()
+    def get_mutation(self) -> MutationOperator[TreeNode]:
+        return DefaultTBMutation()
+
+    def get_crossover(self) -> CrossoverOperator[TreeNode]:
+        return DefaultTBCrossover()
