@@ -1,22 +1,13 @@
 from __future__ import annotations
 
-import random
-from typing import Any
-from typing import Callable
-from typing import Dict
 from typing import get_args
-from typing import Type
 
-import z3
 
 from geneticengine.core.grammar import Grammar
 from geneticengine.core.random.sources import Source
-from geneticengine.core.representations.tree import initialization_methods
-from geneticengine.core.representations.tree.initialization_methods import is_metahandler
+from geneticengine.core.representations.tree import smt
+from geneticengine.core.representations.tree.initializations import is_metahandler
 from geneticengine.metahandlers.base import MetaHandlerGenerator
-from geneticengine.metahandlers.smt.lang import dLit
-from geneticengine.metahandlers.smt.lang import dNode
-from geneticengine.metahandlers.smt.lang import dVar
 from geneticengine.metahandlers.smt.parser import p_expr
 
 
@@ -45,16 +36,16 @@ class SMT(MetaHandlerGenerator):
         c = context.copy()
 
         ident = c["_"]
-        initialization_methods.SMTResolver.register_type(ident, base_type)
+        smt.SMTResolver.register_type(ident, base_type)
 
-        initialization_methods.SMTResolver.add_clause(
+        smt.SMTResolver.add_clause(
             [lambda types: self.restriction.translate(c, types)],
             {},
         )
 
         if base_type == int or base_type == bool or base_type == float:
             # we need the result, add receiver
-            initialization_methods.SMTResolver.add_clause(
+            smt.SMTResolver.add_clause(
                 [],
                 {ident: rec},
             )
